/*
 * BoxedStatements.java       Jan 13, 2007
 *
 * Copyright (c) 2007, Brian Mulanda, all rights reserved.
 */

package edu.ksu.cis.ndpc;

import edu.ksu.cis.ndpc.rules.RuleParameter;

/**
 * Represents the set of statements enclosed within a box in a natural deduction
 * proof statement. Only the statement numbers for the first and last statement
 * within the box are represented, in the form of a range.
 * 
 * References to both the first and last statements in the box are also required
 * 
 * @author Brian Mulanda (<a
 *         href="mailto:bmulanda@ksu.edu">bmulanda@ksu.edu</a>)
 * 
 * @see Range
 * @see ProofStatement
 * @see RuleParameter
 * 
 */

public class BoxedStatements implements RuleParameter {
	private ProofStatement first;
	private ProofStatement last;
	private Range range;

	public BoxedStatements(Range range) {
		this.range = range;
	}

	public BoxedStatements(Range range, ProofStatement first,
			ProofStatement last) {
		this.first = first;
		this.last = last;
		this.range = range;
	}

	public void setFirst(ProofStatement first) {
		this.first = first;
	}

	public void setLast(ProofStatement last) {
		this.last = last;
	}

	public ProofStatement getFirst() {
		return first;
	}

	public ProofStatement getLast() {
		return last;
	}

	public Range getRange() {
		return range;
	}
}
