package edu.ksu.cis.ndpc;

import java.util.HashSet;
import java.util.Hashtable;

/**
 * @author Brian Mulanda
 * 
 */
public class PredLSymbols extends LangSymbols {
	private HashSet pSymbols;
	private HashSet fSymbols;
	private HashSet cSymbols;
	private HashSet sSymbols;
	private HashSet vSymbols;
	private Hashtable symbols;

	public PredLSymbols() {
		symbols = new Hashtable(20);
		pSymbols = new HashSet();
		cSymbols = new HashSet();
		fSymbols = new HashSet();
		sSymbols = new HashSet();
		vSymbols = new HashSet();

		sSymbols.add("(");
		sSymbols.add(")");
		sSymbols.add("v");
		sSymbols.add("V");
		sSymbols.add("^");
		sSymbols.add("~");
		sSymbols.add("->");
		sSymbols.add("<->");
		sSymbols.add("A");
		sSymbols.add("E");
		sSymbols.add(",");
		sSymbols.add("=");
		sSymbols.add("_|_");

		cSymbols.add("c");
		cSymbols.add("d");
		cSymbols.add("e");
		cSymbols.add("m");

		vSymbols.add("a");
		vSymbols.add("b");
		vSymbols.add("t");
		vSymbols.add("u");
		vSymbols.add("w");
		vSymbols.add("x");
		vSymbols.add("y");
		vSymbols.add("z");

		pSymbols.add("F");
		pSymbols.add("G");
		pSymbols.add("H");
		pSymbols.add("P");
		pSymbols.add("Q");
		pSymbols.add("R");
		pSymbols.add("S");

		fSymbols.add("f");
		fSymbols.add("g");
		fSymbols.add("h");
		fSymbols.add("s");
	}

	public boolean isSpecial(String symbol) {
		boolean result = false;
		result = sSymbols.contains(symbol);
		return result;
	}

	public int identify(String symbol) {
		int type = S_INV;
		if (symbol != null) {
			symbol = symbol.trim();
			if (symbol.equals("(")) {
				type = S_OPAR;
			} else if (symbol.equals(")")) {
				type = S_CPAR;
			} else if (symbol.equalsIgnoreCase("v")) {
				type = S_DISJ;
			} else if (symbol.equals("^")) {
				type = S_CONJ;
			} else if (symbol.equals("~")) {
				type = S_NEG;
			} else if (symbol.equals("->")) {
				type = S_IMPL;
			} else if (symbol.equals("<->")) {
				type = S_DIMPL;
			} else if (symbol.equals(",")) {
				type = S_COMMA;
			} else if (symbol.equals("=")) {
				type = S_EQUAL;
			} else if (symbol.equals("A")) {
				type = S_FORALL;
			} else if (symbol.equals("E")) {
				type = S_EXISTS;
			} else if (symbol.equals("_|_")) {
				type = S_BOTTOM;
			} else if (pSymbols.contains(symbol.substring(0, 1))) {
				type = S_PRED;
			} else if (fSymbols.contains(symbol.substring(0, 1))) {
				type = S_FUNC;
			} else if (cSymbols.contains(symbol.substring(0, 1))) {
				type = S_CONST;
			} else if (vSymbols.contains(symbol.substring(0, 1))) {
				type = S_VAR;
			}
		}
		return type;
	}

	public boolean isFunction(int type) {
		return type == S_FUNC;
	}

	public boolean isFunction(Symbol symbol) {
		boolean result = false;
		if (symbol != null) {
			result = isFunction(symbol.getType());
		}
		return result;
	}

	public boolean isPredicate(int type) {
		return type == S_PRED;
	}

	public boolean isPredicate(Symbol symbol) {
		boolean result = false;
		if (symbol != null) {
			result = isPredicate(symbol.getType());
		}
		return result;
	}

	public boolean isConstant(int type) {
		return type == S_CONST;
	}

	public boolean isConstant(Symbol symbol) {
		boolean result = false;
		if (symbol != null) {
			result = isConstant(symbol.getType());
		}
		return result;
	}

	public boolean isVariable(int type) {
		return type == S_VAR;
	}

	public boolean isConnective(int type) {
		return (type == S_DISJ) || (type == S_CONJ) || (type == S_IMPL)
				|| (type == S_DIMPL);
	}

	public boolean isConnective(Symbol symbol) {
		boolean result = false;
		if (symbol != null) {
			result = isConnective(symbol.getType());
		}
		return result;
	}

	public boolean isQuantifier(int type) {
		return (type == S_EXISTS) || (type == S_FORALL);
	}

	public boolean isQuantifier(Symbol symbol) {
		boolean result = false;
		if (symbol != null) {
			result = isQuantifier(symbol.getType());
		}
		return result;
	}

	public boolean isTerm(String symbol) {
		boolean result = false;
		if (symbol != null) {
			result = cSymbols.contains(symbol) || fSymbols.contains(symbol)
					|| vSymbols.contains(symbol);
		}
		return result;
	}

	public boolean isTerm(Symbol symbol) {
		boolean result = false;
		if (symbol != null) {
			result = isTerm(symbol.getValue());
		}
		return result;
	}

	public static boolean isTerm(int type) {
		boolean result = false;
		result = type == S_FUNC || type == S_VAR || type == S_CONST;
		return result;
	}

	public boolean isOperator(Symbol symbol) {
		if (symbol != null) {
			return isOperator(symbol.getType());
		} else {
			return false;
		}
	}

	public boolean isOperator(int type) {
		boolean retval = false;
		retval = (type == S_DISJ) || (type == S_CONJ) || (type == S_IMPL)
				|| (type == S_DIMPL) || (type == S_NEG) || (type == S_EQUAL)
				|| (type == S_FORALL) || (type == S_EXISTS);
		return retval;
	}

	public String[] getSpecialSymbols() {
		return (String[]) sSymbols.toArray(new String[sSymbols.size()]);
	}

	public int getAssociativity(int opSymbolType) {
		return ASSOC_RIGHT;
	}

	public int getRelativePrecedence(int opSymbol1Type, int opSymbol2Type) {
		int precedence = PRECE_EQUAL;
		if (opSymbol1Type == opSymbol2Type) {
			int assoc = getAssociativity(opSymbol1Type);
			switch (assoc) {
			case ASSOC_LEFT:
				precedence = PRECE_HIGHER;
				break;
			case ASSOC_RIGHT:
				precedence = PRECE_LOWER;
				break;
			}
		} else {
			if (getPrecedence(opSymbol1Type) < getPrecedence(opSymbol2Type)) {
				precedence = PRECE_HIGHER;
			} else if (getPrecedence(opSymbol1Type) > getPrecedence(opSymbol2Type)) {
				precedence = PRECE_LOWER;
			} else {
				precedence = PRECE_EQUAL;
			}
		}
		return precedence;
	}

	public int getPrecedence(int opType) {
		int precedence = Integer.MAX_VALUE;
		if (opType != S_NULL) {
			if (opType == S_EQUAL) {
				precedence = 1;
			} else if (opType == S_NEG) {
				precedence = 2;
			} else if (opType == S_EXISTS) {
				precedence = 2;
			} else if (opType == S_FORALL) {
				precedence = 2;
			} else if (opType == S_CONJ) {
				precedence = 3;
			} else if (opType == S_DISJ) {
				precedence = 3;
			} else if (opType == S_IMPL) {
				precedence = 4;
			} else if (opType == S_DIMPL) {
				precedence = 4;
			}
		}
		return precedence;
	}

	public boolean exists(String symbol) {
		return (symbol != null) && symbols.containsKey(symbol);
	}

	public boolean addSymbol(Symbol symbol) {
		return addSymbol(symbol, 0);
	}

	public boolean addSymbol(Symbol symbol, int arity) {
		boolean added = false;
		if (symbol != null
				&& (symbol.getType() == S_VAR || symbol.getType() == S_FUNC || symbol
						.getType() == S_PRED)) {
			if (!symbols.containsKey(symbol.getValue())) {
				symbols.put(symbol.getValue(), new Integer(arity));
				added = true;
			}
		}
		return added;
	}

	public int getArity(Symbol symbol) {
		int arity = -1;
		if (symbols != null
				&& (symbol.getType() == S_FUNC || symbol.getType() == S_PRED)) {
			Integer intArity = (Integer) symbols.get(symbol.getValue());
			if (intArity != null) {
				arity = intArity.intValue();
			}
		}
		return arity;
	}

	public void forget(String symbol) {
		if (symbols.containsKey(symbol)) {
			symbols.remove(symbol);
		}
	}
}