/*
 * ProofFormula.java       Jan 13, 2007
 *
 * Copyright (c) 2007, Brian Mulanda, all rights reserved.
 */

package edu.ksu.cis.ndpc;

/**
 * Represents a parsed natural deduction proof formula.
 * 
 *
 * @author Brian Mulanda
 *         (<a href="mailto:bmulanda@ksu.edu">bmulanda@ksu.edu</a>)
 * 
 * @see ParseTree
 */

import java.util.ArrayList;

public class ProofFormula {
	public String formula;
	public ParseTree tree;
	public ArrayList variables;

	public ProofFormula(String formula) {
		this(formula, null, null);
	}

	public ProofFormula(String formula, ParseTree tree) {
		this(formula, tree, null);
	}

	public ProofFormula(String formula, ParseTree tree, ArrayList variables) {
		this.formula = formula;
		this.tree = tree;
		this.variables = variables;
	}

	public void setParseTree(ParseTree tree) {
		this.tree = tree;
	}

	public ParseTree getParseTree() {
		return tree;
	}

	public void setVariables(ArrayList variables) {
		this.variables = variables;
	}

	public ArrayList getVariables() {
		return variables;
	}

	public String getFormulaString() {
		return formula;
	}

	public String toString() {
		return formula;
	}

	public boolean equals(Object obj) {
		boolean equal = false;
		try {
			if (obj instanceof ProofFormula) {
				ProofFormula f = (ProofFormula) obj;
				if (tree != null && f.tree != null) {
					equal = tree.equals(f.tree);
				}
			}
		} catch (Exception e) {

		}
		return equal;
	}

	public boolean usesVariable(Symbol variable) {
		return (variables != null && variables.contains(variable));
	}
}