/*
 * ProofStatement.java       Jan 13, 2007
 *
 * Copyright (c) 2007, Brian Mulanda, all rights reserved.
 */

package edu.ksu.cis.ndpc;

import edu.ksu.cis.ndpc.rules.Rule;
import edu.ksu.cis.ndpc.rules.RuleParameter;

/**
 * Represents a parsed, but unchecked, natural deduction proof statement.
 * 
 * 
 * @author Brian Mulanda (<a
 *         href="mailto:bmulanda@ksu.edu">bmulanda@ksu.edu</a>)
 * 
 * @see Rule
 * @see LineReference
 * @see ProofFormula
 * @see RuleParameter
 * 
 */

public class ProofStatement implements RuleParameter {

	private int lineno = 0;
	public ProofVariable variable = null;
	public ProofFormula formula = null;
	public ProofRule rule = null;
	public ProofLineReference lineref = null;

	public ProofStatement(int lineno) {
		this(lineno, null, null, null, null);
	}

	public ProofStatement(int lineno, ProofVariable variable) {
		this(lineno, variable, null, null, null);
	}

	public ProofStatement(int lineno, ProofFormula formula, ProofRule rule) {
		this(lineno, null, formula, rule, null);
	}

	public ProofStatement(int lineno, ProofVariable variable,
			ProofFormula formula, ProofRule rule) {
		this(lineno, variable, formula, rule, null);
	}

	public ProofStatement(int lineno, ProofVariable variable,
			ProofFormula formula, ProofRule rule, ProofLineReference lineref) {
		this.lineno = lineno;
		if (variable != null) {
			this.variable = variable;
		}
		this.formula = formula;
		this.rule = rule;
		this.lineref = lineref;
	}

	public int getLineNo() {
		return lineno;
	}

	public ProofVariable getVariable() {
		return variable;
	}

	public ProofFormula getFormula() {
		return formula;
	}

	public ProofRule getRule() {
		return rule;
	}

	public ProofLineReference getLineref() {
		return lineref;
	}

	public String toString() {
		/*
		 * return lineno + ". " + ((variable != null) ? variable.getValue() :
		 * "") + " " + formula;
		 */
		return lineno + ". " + formula;
	}
}
