package edu.ksu.cis.ndpc;

import java.io.IOException;
import java.io.StreamTokenizer;
import java.io.StringReader;

/**
 * @author Brian Mulanda
 * 
 */
public class Tokenizer {
	private StreamTokenizer tokenizer;
	private Token token;
	private Token nextToken;
	private int position;

	public Tokenizer(String formula, String ordChar) {
		StringReader strReader = new StringReader(formula);
		tokenizer = new StreamTokenizer(strReader);
		if (ordChar != null) {
			char[] chars = ordChar.toCharArray();
			for (int i = 0; i < chars.length; ++i) {
				tokenizer.ordinaryChar(chars[i]);
			}
		}
		position = 0;
		nextToken = getNextToken();
	}

	public Tokenizer(String formula, String[] ordChar) {
		StringReader strReader = new StringReader(formula);
		tokenizer = new StreamTokenizer(strReader);
		if (ordChar != null) {
			for (int i = 0; i < ordChar.length; ++i) {
				if (ordChar[i] != null && ordChar[i].length() > 0) {
					char[] chars = ordChar[i].toCharArray();
					for (int j = 0; j < chars.length; ++j) {
						tokenizer.ordinaryChar(chars[j]);
					}
				}
			}
		}
		position = 0;
		nextToken = getNextToken();
	}

	public boolean nextToken() {
		boolean validToken = (nextToken != null);
		if (validToken) {
			token = nextToken;
			nextToken = getNextToken();
		}
		return (validToken);
	}

	public Token getToken() {
		return token;
	}

	private Token getNextToken() {
		Token t = null;
		String tokenString = getTokenString();
		if (tokenString != null) {
			int length = tokenString.length();
			t = new Token(tokenString.trim(), position, length);
			position += length;
		} else {
			t = null;
		}
		return t;
	}

	private String getTokenString() {
		String token = null;
		boolean done = false;

		try {
			while (!done) {
				done = true;
				tokenizer.nextToken();
				switch (tokenizer.ttype) {
				case StreamTokenizer.TT_EOF:
					break;
				case StreamTokenizer.TT_EOL:
					token = getTokenString();
					break;
				case StreamTokenizer.TT_NUMBER:
					token = Integer.toString(new Double(tokenizer.nval)
							.intValue());
					break;
				case StreamTokenizer.TT_WORD:
					token = tokenizer.sval;
					break;
				default:
					token = Character.toString((char) tokenizer.ttype);
					if (token.equals(" ")) {
						done = false;
					}
					break;
				}
			}
		} catch (IOException e) {

		}
		return token;
	}

	public void pushBack() {
		if (token != null) {
			tokenizer.pushBack();
			nextToken = token;
		}
	}
}
