package edu.ksu.cis.ndpc.gui;

import java.awt.Dimension;
import edu.ksu.cis.ndpc.Utils;

public class PredGUIController extends GUIController {
	private static final int COL_LNUM = 0;
	private static final int COL_VAR = 1;
	private static final int COL_FORM = 2;
	private static final int COL_RULE = 3;
	private static final int COL_LREF = 4;

	private static final int[] DEF_MIN_ROWSIZE = { 4, 3, 20, 8, 8 };
	private int[] minRowsize;
	private int[] colWidth;
	private double[] colWidthFrac = { 0.05, 0.10, 0.45, 0.20, 0.20 };

	protected int getVariableColumn() {
		return COL_VAR;
	}

	protected int getFormulaColumn() {
		return COL_FORM;
	}

	protected int getLinerefColumn() {
		return COL_LREF;
	}

	protected void setupColumns() {
		int columnCount = 5;
		content = new GUIContent(columnCount, 1);
		colWidth = new int[columnCount];
		for (int i = 0; i < columnCount; ++i) {
			colWidth[i] = 0;
		}
		content.getCol(COL_LNUM).getRow(0).append("1.");
	}

	protected void goToFirstCol() {
		cursor.col = COL_VAR;
	}

	protected void goToLastCol() {
		cursor.col = COL_LREF;
	}

	protected void goToFormulaCol() {
		cursor.col = COL_FORM;
	}

	protected boolean isLastCol(int col) {
		return col == COL_LREF;
	}

	protected boolean isFirstCol(int col) {
		return col == COL_VAR;
	}

	public RawProof getProof() {
		Col cv = content.getCol(COL_VAR);
		Col cf = content.getCol(COL_FORM);
		Col cr = content.getCol(COL_RULE);
		Col cl = content.getCol(COL_LREF);

		int rows = content.getRowCount();
		RawProof proof = new RawProof(RawProof.PROOF_PRED);
		for (int row = 0; row < rows; ++row) {
			RawStatement statement = new RawStatement();
			// RawStatement statement = null;
			if (content.getRowLength(row, true) > 0) {
				// statement = new RawStatement();
				statement.setVariable(cv.getRow(row).toString());
				statement.setFormula(Utils.replaceAll(
						cf.getRow(row).toString(), "_", "_|_"));
				statement.setRule(Utils.replaceAll(cr.getRow(row).toString(),
						"_", "_|_"));
				statement.setLineRef(cl.getRow(row).toString());
			}
			proof.addStatement(statement);
		}
		for (int i = 0; i < boxMan.boxCount(); ++i) {
			GUIBox uibox = boxMan.get(i);
			RawBox rbox = new RawBox(uibox.startRow(), uibox.endRow() - 1);
			proof.addBox(rbox);
		}
		return proof;
	}

	protected void setupProof(RawProof proof) {
		content.removeAllRows();
		if (proof != null) {
			RawStatement statement = null;
			int rowCount = proof.size();
			// System.out.println("Proof size=" + rowCount);
			content.appendRows(rowCount);

			Col cln = content.getCol(COL_LNUM);
			Col cv = content.getCol(COL_VAR);
			Col cf = content.getCol(COL_FORM);
			Col cr = content.getCol(COL_RULE);
			Col cl = content.getCol(COL_LREF);

			minRowsize = new int[DEF_MIN_ROWSIZE.length];
			for (int i = 0; i < DEF_MIN_ROWSIZE.length; ++i) {
				minRowsize[i] = DEF_MIN_ROWSIZE[i];
			}

			for (int i = 0; i < rowCount; ++i) {
				statement = proof.getStatement(i);
				String item = null;
				cln.getRow(i).setData(Integer.toString(i + 1));
				if (statement != null) {
					item = statement.getVariable();
					if (item != null) {
						cv.getRow(i).setData(item);
						if (item.length() > minRowsize[COL_VAR]) {
							minRowsize[COL_VAR] = item.length();
						}
					}

					item = statement.getFormula();
					if (item != null) {
						item = Utils.replaceAll(item, "_|_", "_");
						cf.getRow(i).setData(item);
						if (item.length() > minRowsize[COL_FORM]) {
							minRowsize[COL_FORM] = item.length();
						}
					}

					item = statement.getRule();
					if (item != null) {
						item = Utils.replaceAll(item, "_|_", "_");
						cr.getRow(i).setData(item);
						if (item.length() > minRowsize[COL_RULE]) {
							minRowsize[COL_RULE] = item.length();
						}
					}

					item = statement.getLineref();
					if (item != null) {
						cl.getRow(i).setData(item);
						if (item.length() > minRowsize[COL_LREF]) {
							minRowsize[COL_LREF] = item.length();
						}
					}
				}
			}

			for (int i = 1; i < minRowsize.length; ++i) {
				minRowsize[i] += cellPadding;
			}
			recalcRowFrac();
		}
	}

	private void recalcRowFrac() {
		double rowsize = getRowsize();
		for (int i = 0; i < minRowsize.length; ++i) {
			colWidthFrac[i] = minRowsize[i] / rowsize;
		}
	}

	protected Boundary calcBoxBounds(int startRow, int endRow, int h) {
		int rowCount = 0;
		int width = 0;
		int x = 0;
		int y = 0;
		int height = 0;

		if (startRow > endRow) {
			rowCount = startRow - endRow;
			y += margin.top + (getRowHeight() * startRow);
			y -= boxPixels + boxHPixels + rowSpacing;
			height = -getBoxHeight(rowCount);
		} else {
			rowCount = endRow - startRow;
			y += margin.top + (getRowHeight() * startRow) + boxHPixels;
			height = getBoxHeight(rowCount);
		}

		x += margin.left + getColWidth(COL_LNUM)
				+ getColSpacing(COL_LNUM, COL_VAR);
		int spacing = (h * (boxPixels + boxHPixels + BoxHandler.getSpacing()));

		x -= (spacing + boxPixels + boxHPixels);
		x += boxHPixels;

		for (int i = COL_VAR; i < content.getColumnCount(); ++i) {
			width += getColWidth(i) + getColSpacing(i, i + 1);
		}
		width += (h * (boxPixels + boxHPixels + BoxHandler.getSpacing()) * 2);
		width += boxPixels;

		return new Boundary(x, y, x + width, y + height);
	}

	protected int getColWidth(int col) {
		if (col >= 0 && col < content.getColumnCount()) {
			return colWidth[col];
		} else {
			return 0;
		}
	}

	protected void resizeColumn(int col, int pixels) {
		colWidth[col] += pixels;
		Dimension d = gui.getPreferredSize();
		gui.setPreferredSize(new Dimension(d.width + pixels, d.height));
		recalcBoxBounds();
		recalcColWidthFracs();
	}

	private void recalcColWidthFracs() {
		int width = gui.getPreferredSize().width;
		for (int i = 0; i < colWidth.length; ++i) {
			colWidthFrac[i] = (colWidth[i] * 1.0 / width);
		}
	}

	protected int getColSpacing(int col1, int col2) {
		int diff = col2 - col1;
		int spacing = 0;
		for (int i = 0; i < diff; ++i) {
			col1 = col1 + i;
			if (col1 == COL_LNUM) {
				spacing += colSpacing;
				if (boxMan.height() >= 2) {
					spacing += ((boxMan.height() - 1) * BoxHandler.getSpacing());
				}
				spacing += (boxMan.height() * (boxPixels + boxHPixels));
			} else if (!isLastCol(col1)) {
				spacing += colSpacing;
			}
		}
		return spacing;
	}

	protected void resizeColumns() {
		int guiWidth = gui.getPreferredSize().width;
		if (guiWidth > 100) {
			int width = guiWidth;
			width -= (margin.left + margin.right + (colSpacing * (content
					.getColumnCount() - 1)));
			if (boxMan.height() >= 2) {
				width -= ((boxMan.height() - 1) * BoxHandler.getSpacing() * 2);
			}
			width -= ((boxMan.height() * (boxPixels + boxHPixels)) * 2);

			// System.out.println("setColWidths(): width: " + width);
			colWidth[0] = (int) Math.floor(width * colWidthFrac[0]);
			colWidth[1] = (int) Math.floor(width * colWidthFrac[1]);
			colWidth[3] = (int) Math.floor(width * colWidthFrac[3]);
			colWidth[4] = (int) Math.floor(width * colWidthFrac[4]);

			colWidth[2] = width
					- (colWidth[0] + colWidth[1] + colWidth[3] + colWidth[4]);

			colWidthFrac[2] = (colWidth[2] * 1.0) / guiWidth;

			recalcBoxBounds();
		}
	}

	protected int getRowsize() {
		int rowsize = 0;
		for (int i = 0; i < minRowsize.length; ++i) {
			rowsize += minRowsize[i];
		}
		return rowsize;
	}

	protected int getRowsize(int col) {
		return minRowsize[col];
	}

	protected void setRowsize(int col, int size) {
		if (size < DEF_MIN_ROWSIZE[col]) {
			minRowsize[col] = DEF_MIN_ROWSIZE[col];
		} else {
			minRowsize[col] = size;
		}
		recalcRowFrac();
	}

	protected void resetRowsize(int col) {
		int size = DEF_MIN_ROWSIZE[col];
		Col c = content.getCol(col);
		int rowlength = 0;
		for (int i = 0; i < c.rowCount(); ++i) {
			rowlength = c.getRow(i).length();
			if (rowlength > size)
				size = rowlength;
		}
		minRowsize[col] = size + cellPadding;
		recalcRowFrac();
	}

	protected void updateLineNums(int startRow) {
		Col col = content.getCol(COL_LNUM);
		updateLineNums(startRow, col.rowCount() - 1);
	}

	protected void updateLineNums(int startRow, int endRow) {
		Col col = content.getCol(COL_LNUM);
		for (int i = startRow; i <= endRow; ++i) {
			col.getRow(i).setData(Integer.toString(i + 1));
		}
	}
}