/*
 * RawBox.java       Jan 19, 2007
 *
 * Copyright (c) 2007, Brian Mulanda, all rights reserved.
 */

package edu.ksu.cis.ndpc.gui;

import java.io.Serializable;

/**
 * Represents a box in the proof as it appears on the graphical user interface.
 * This box is serialized when a proof is saved to a file in proprietary format.
 * 
 * 
 * @author Brian Mulanda (<a
 *         href="mailto:bmulanda@ksu.edu">bmulanda@ksu.edu</a>)
 * 
 * @see RawStatement
 * @see RawProof
 * @see java.io.Serializable
 * 
 */

public class RawBox implements Serializable {
	/**
	 * The class fingerprint that is set to indicate serialization compatibility
	 * with a previous version of the class.
	 */
	static final long serialVersionUID = 1;

	private int startRow;
	private int endRow;

	public RawBox(int startRow, int endRow) {
		this.startRow = startRow;
		this.endRow = endRow;
	}

	public int startRow() {
		return startRow;
	}

	public int endRow() {
		return endRow;
	}

	public String toString() {
		return Integer.toString(startRow) + "," + Integer.toString(endRow);
	}

	public static RawBox fromString(String s) {
		RawBox box = null;
		if (s != null) {
			try {
				String[] boxArr = s.split(",");
				if (boxArr.length >= 2) {
					int start = Integer.parseInt(boxArr[0].trim());
					int end = Integer.parseInt(boxArr[1].trim());
					box = new RawBox(start, end);
				}
			} catch (Exception e) {
				box = null;
			}
		}
		return box;
	}
}