/*
 * RawStatement.java       Jan 19, 2007
 *
 * Copyright (c) 2007, Brian Mulanda, all rights reserved.
 */

package edu.ksu.cis.ndpc.gui;

import java.io.Serializable;

/**
 * Represents a statement in the proof as it appears on the graphical user
 * interface. This statement is serialized when a proof is saved to a file in
 * proprietary format.
 * 
 * 
 * @author Brian Mulanda (<a
 *         href="mailto:bmulanda@ksu.edu">bmulanda@ksu.edu</a>)
 * 
 * @see RawBox
 * @see RawProof
 * @see java.io.Serializable
 * 
 */

public class RawStatement implements Serializable {
	/**
	 * The class fingerprint that is set to indicate serialization compatibility
	 * with a previous version of the class.
	 */
	static final long serialVersionUID = 1;

	private transient static final String BLANK = "_";
	private transient static final String ITEM_DELIM = ":";

	private String variable;
	private String formula;
	private String rule;
	private String lineref;

	public RawStatement() {
		variable = null;
		formula = null;
		rule = null;
		lineref = null;
	}

	public void setVariable(String variable) {
		if (variable != null && variable.trim().length() > 0) {
			this.variable = variable.trim();
		}
	}

	public void setFormula(String formula) {
		if (formula != null && formula.trim().length() > 0) {
			this.formula = formula.trim();
		}
	}

	public void setRule(String rule) {
		if (rule != null && rule.trim().length() > 0) {
			this.rule = rule.trim();
		}
	}

	public void setLineRef(String lineref) {
		if (lineref != null && lineref.trim().length() > 0) {
			this.lineref = lineref.trim();
		}
	}

	public String getVariable() {
		return variable;
	}

	public String getFormula() {
		return formula;
	}

	public String getRule() {
		return rule;
	}

	public String getLineref() {
		return lineref;
	}

	public String toString() {
		StringBuffer statement = new StringBuffer();
		statement.append((variable == null ? BLANK : variable)).append(
				ITEM_DELIM);
		statement.append((formula == null ? BLANK : formula))
				.append(ITEM_DELIM);
		statement.append((rule == null ? BLANK : rule)).append(ITEM_DELIM);
		statement.append((lineref == null ? BLANK : lineref));
		return statement.toString();
	}

	public static RawStatement fromString(String s) {
		RawStatement statement = null;
		if (s != null) {
			try {
				String[] arr = s.split(ITEM_DELIM);
				if (arr.length >= 4) {
					statement = new RawStatement();
					String item = null;
					int i = 0;
					item = arr[i++];
					if (!item.equals(BLANK))
						statement.setVariable(item);
					item = arr[i++];
					if (!item.equals(BLANK))
						statement.setFormula(item);
					item = arr[i++];
					if (!item.equalsIgnoreCase(BLANK))
						statement.setRule(item);
					item = arr[i++];
					if (!item.equalsIgnoreCase(BLANK))
						statement.setLineRef(item);
				}
			} catch (Exception e) {
				statement = null;
			}
		}
		return statement;
	}
}