/*
 * InputFile.java       Jan 19, 2007
 *
 * Copyright (c) 2007, Brian Mulanda, all rights reserved.
 */

package edu.ksu.cis.ndpc.io;

import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

/**
 * Represents an file opened for input
 * 
 * @author Brian Mulanda (<a
 *         href="mailto:bmulanda@ksu.edu">bmulanda@ksu.edu</a>)
 * 
 * @see InputFile
 * @see IO
 * 
 */

public class InputFile {
	private FileInputStream fis;
	private File f;

	public InputFile(File f) throws IOException {
		try {
			this.f = f;
			fis = new FileInputStream(f);
		} catch (Exception e) {
			throw new IOException(e.getMessage());
		}
	}

	public File getFile() {
		return f;
	}

	public String getName() {
		return f.getName();
	}

	public String getAbsolutePath() {
		return f.getAbsolutePath();
	}

	public Object readObject() throws IOException {
		Object o = null;
		try {
			ObjectInputStream ois = new ObjectInputStream(fis);
			o = ois.readObject();
		} catch (Exception e) {
			throw new IOException(e.getMessage());
		}
		return o;
	}

	public String readSting() throws IOException {
		String s = null;
		try {
			BufferedReader bfr = new BufferedReader(new InputStreamReader(fis));
			char[] data = new char[5000000];
			int charCount = bfr.read(data);
			if (charCount > 0) {
				s = new String(data, 0, charCount);
			}
		} catch (Exception e) {
			throw new IOException(e.getMessage());
		}
		return s;
	}

	public void close() {
		finalize();
	}

	public void finalize() {
		try {
			fis.close();
		} catch (IOException ioe) {
		} finally {
			fis = null;
		}
	}
}