package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleDbNegElim extends Rule {
	private static String name = getDescription(Rule.DNEG_ELIM);

	public RuleDbNegElim() {
		super();
	}

	public int getType() {
		return Rule.DNEG_ELIM;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 2 || params[0] == null
				|| params[1] == null) {
			throw new RuleException("Rule requires a formula");
		} else if (!((params[0] instanceof ProofStatement) && (params[1] instanceof ProofStatement))) {
			throw new RuleException("Rule requires a formula");
		}

		ProofStatement premise = (ProofStatement) params[0];
		ProofStatement conclusion = (ProofStatement) params[1];

		Symbol root = null;
		root = premise.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_NEG) {
			throw new RuleException("Formula " + premise
					+ " must negated twice (begin with two '~')");
		}

		root = premise.getFormula().getParseTree().getSubtree().getRootNode();
		if (root.getType() != LangSymbols.S_NEG) {
			throw new RuleException("Formula " + premise
					+ " must negated twice (begin with two '~')");
		}

		ParseTree subtree = premise.getFormula().getParseTree().getSubtree()
				.getSubtree();
		if (!subtree.equals(conclusion.getFormula().getParseTree())) {
			throw new RuleException(
					"Formula "
							+ conclusion
							+ " must be such that if you "
							+ "negate it twice: ~(~(formula)) or ~~formula or ~(~formula), "
							+ "the resulting formula must be the same as formula"
							+ premise);
		}
	}
}