package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofChecker;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleEqualsElim extends Rule {
	private static String name = getDescription(Rule.EQUAL_ELIM);

	public RuleEqualsElim() {
		super();
	}

	public int getType() {
		return Rule.EQUAL_ELIM;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 3 || params[0] == null
				|| params[1] == null || params[2] == null) {
			throw new RuleException("Rule requires two formulas");
		} else if (!((params[0] instanceof ProofStatement)
				&& (params[1] instanceof ProofStatement) && (params[2] instanceof ProofStatement))) {
			throw new RuleException("Rule requires two formulas");
		}

		ProofStatement premise1 = (ProofStatement) params[0];
		ProofStatement premise2 = (ProofStatement) params[1];
		ProofStatement conclusion = (ProofStatement) params[2];

		Symbol root = premise1.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_EQUAL) {
			throw new RuleException("Formula " + premise1
					+ " must have '=' as its main connective");
		}

		ParseTree termX = premise1.getFormula().getParseTree().getLeftSubtree();
		ParseTree termT = premise1.getFormula().getParseTree()
				.getRightSubtree();

		if (termX.equals(termT)) {
			if (!premise2.getFormula().getParseTree().equals(
					conclusion.getFormula().getParseTree())) {
				throw new RuleException(
						" Since the terms on either side of the main connective '=' "
								+ "in " + premise1
								+ " are the same, then the formula "
								+ conclusion
								+ " should be the same as the formula "
								+ premise1);
			}
		} else {
			ParseTree conclTestTree = null;
			ParseTree prem2TestTree = null;

			if (termT.size() >= termX.size()) {
				ParseTree temp = termT;
				termT = termX;
				termX = temp;
			}
			conclTestTree = ProofChecker.substitute((ParseTree) conclusion
					.getFormula().getParseTree().clone(), termX, termT);
			while (!conclTestTree.equals(termX)
					&& ProofChecker.countFreeIn(termX, conclTestTree) > 0) {
				conclTestTree = ProofChecker.substitute(conclTestTree, termX,
						termT);
			}

			prem2TestTree = ProofChecker.substitute((ParseTree) premise2
					.getFormula().getParseTree().clone(), termX, termT);
			while (!prem2TestTree.equals(termX)
					&& ProofChecker.countFreeIn(termX, prem2TestTree) > 0) {
				prem2TestTree = ProofChecker.substitute(prem2TestTree, termX,
						termT);
			}

			if (!conclTestTree.equals(prem2TestTree)) {
				throw new RuleException("Incorrect substitution.\n");
			}
		}
	}
}