package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.BoxedStatements;
import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofChecker;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleExistsElim extends Rule {
	private static String name = getDescription(Rule.EXIST_ELIM);

	public RuleExistsElim() {
		super();
	}

	public int getType() {
		return Rule.EXIST_ELIM;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 3 || params[0] == null
				|| params[1] == null || params[2] == null) {
			throw new RuleException("Rule requires one set of boxed "
					+ " formulas and one unboxed formula");
		} else if (!((params[0] instanceof ProofStatement)
				&& (params[1] instanceof BoxedStatements) && (params[2] instanceof ProofStatement))) {
			throw new RuleException("Rule requires one set of boxed "
					+ " formulas and one unboxed formula");
		}

		ProofStatement premise1 = (ProofStatement) params[0];
		BoxedStatements premise2 = (BoxedStatements) params[1];
		ProofStatement conclusion = (ProofStatement) params[2];

		try {
			ParseTree formula1Tree = premise1.getFormula().getParseTree();
			ParseTree formula2Tree = premise2.getFirst().getFormula()
					.getParseTree();
			ParseTree formula3Tree = premise2.getLast().getFormula()
					.getParseTree();
			ParseTree formula4Tree = conclusion.getFormula().getParseTree();
			ParseTree variable = null;

			if (formula1Tree.getRootNode().getType() != LangSymbols.S_EXISTS) {
				throw new RuleException(
						"Formula "
								+ premise1
								+ " must be existentially quantified "
								+ "with the existential symbol and a variable. For example Ex(formula)");
			}

			Symbol freshVar = premise2.getFirst().getVariable();
			if (freshVar != null && freshVar.getValue() != null) {
				variable = new ParseTree(freshVar);
			} else {
				throw new RuleException(
						"You must introduce a fresh variable in line "
								+ premise2.getFirst().getLineNo() + ".");
			}

			if (premise2.getFirst().getRule().type != Rule.ASSUMPTION) {
				throw new RuleException("The first formula "
						+ premise2.getFirst() + " " + "in box "
						+ premise2.getRange() + " must be an " + "assumption");
			}

			Symbol var = new Symbol(formula1Tree.getRootNode().getValue()
					.substring(1), LangSymbols.S_VAR);
			ParseTree termX = new ParseTree(var);
			ParseTree termT = variable;

			ParseTree testAssumpTree = ProofChecker
					.substitute((ParseTree) formula1Tree.getSubtree().clone(),
							termX, termT);

			if (!testAssumpTree.equals(formula2Tree)) {
				String formula = premise1.getFormula().getFormulaString()
						.substring(0, formula1Tree.getRootNode().getLength())
						.trim();

				throw new RuleException("Incorrect substitution in "
						+ premise2.getFirst()
						+ ". The correct assumption is obtained from "
						+ premise1 + " by first " + "removing " + formula
						+ ", and then replacing all free occurrences " + "of "
						+ termX + " with " + variable + ".");
			}

			if (!formula3Tree.equals(formula4Tree)) {
				throw new RuleException("Formula " + conclusion
						+ " must be same as Formula " + premise2.getLast());
			}
		} catch (RuleException re) {
			throw re;
		} catch (Exception e) {
			throw new RuleException(e.getMessage());
		}
	}
}