package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.BoxedStatements;
import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleImpIntro extends Rule {
	private static String name = getDescription(Rule.IMP_INTRO);

	public RuleImpIntro() {
		super();
	}

	public int getType() {
		return Rule.IMP_INTRO;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 2 || params[0] == null
				|| params[1] == null) {
			throw new RuleException("Rule requires one set of boxed formulas");
		} else if (!((params[0] instanceof BoxedStatements) && (params[1] instanceof ProofStatement))) {
			throw new RuleException("Rule requires one set of boxed formulas");
		}

		BoxedStatements premise = (BoxedStatements) params[0];
		ProofStatement conclusion = (ProofStatement) params[1];

		Symbol root = conclusion.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_IMPL) {
			throw new RuleException("Formula " + conclusion
					+ " must have '->' as its main connective");
		}

		ProofStatement firstFormula = premise.getFirst();
		ProofStatement lastFormula = premise.getLast();
		ParseTree cRightSubtree = conclusion.getFormula().getParseTree()
				.getRightSubtree();
		ParseTree cLeftSubtree = conclusion.getFormula().getParseTree()
				.getLeftSubtree();

		if (firstFormula.getRule().type != Rule.ASSUMPTION) {
			throw new RuleException("The first formula " + firstFormula + " "
					+ "in box " + premise.getRange() + " must be an "
					+ "assumption");
		}

		if (!cLeftSubtree.equals(firstFormula.getFormula().getParseTree())) {
			throw new RuleException("The formula on the left side of the main "
					+ "connective '->' in formula " + conclusion + " must be "
					+ "the same as the first formula " + firstFormula
					+ " in the " + "box: " + premise.getRange());
		} else if (!cRightSubtree.equals(lastFormula.getFormula()
				.getParseTree())) {
			throw new RuleException(
					"The formula on the right side of the main "
							+ "connective '->' in formula " + conclusion
							+ " must be " + "the same as the last formula "
							+ lastFormula + " in the " + "box: "
							+ premise.getRange());
		}
	}
}