package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleNegElim extends Rule {
	private static String name = getDescription(Rule.NEG_ELIM);

	public RuleNegElim() {
		super();
	}

	public int getType() {
		return Rule.NEG_ELIM;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 3 || params[0] == null
				|| params[1] == null || params[2] == null) {
			throw new RuleException("Rule requires two formulas");
		} else if (!((params[0] instanceof ProofStatement)
				&& (params[1] instanceof ProofStatement) && (params[2] instanceof ProofStatement))) {
			throw new RuleException("Rule requires two formulas");
		}

		ProofStatement premise1 = (ProofStatement) params[0];
		ProofStatement premise2 = (ProofStatement) params[1];
		ProofStatement conclusion = (ProofStatement) params[2];

		Symbol root = premise2.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_NEG) {
			throw new RuleException("Formula " + premise2
					+ " must be negated: ~formula or ~(formula)");
		}

		ParseTree subtree = premise2.getFormula().getParseTree().getSubtree();
		if (!subtree.equals(premise1.getFormula().getParseTree())) {
			throw new RuleException(
					"Formula "
							+ premise2
							+ " must be such that "
							+ "if you remove the first negation, the resulting formula "
							+ "must be the same as formula " + premise1);
		}

		root = conclusion.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_BOTTOM) {
			throw new RuleException("Formula " + conclusion
					+ " must be bottom: _|_");
		}
	}
}