package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.BoxedStatements;
import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleOrElim extends Rule {
	private static String name = getDescription(Rule.OR_ELIM);

	public RuleOrElim() {
		super();
	}

	public int getType() {
		return Rule.OR_ELIM;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 4 || params[0] == null
				|| params[1] == null || params[2] == null || params[3] == null) {
			throw new RuleException("Rule requires two sets of boxed "
					+ " formulas and an unboxed formula");
		} else if (!((params[0] instanceof ProofStatement)
				&& (params[1] instanceof BoxedStatements)
				&& (params[2] instanceof BoxedStatements) && (params[3] instanceof ProofStatement))) {
			throw new RuleException("Rule requires two sets of boxed "
					+ " formulas and an unboxed formula");
		}

		ProofStatement premise1 = (ProofStatement) params[0];
		Symbol root = premise1.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_DISJ) {
			throw new RuleException("Formula " + premise1
					+ " must have 'v' as its main connective");
		}

		BoxedStatements premise2 = (BoxedStatements) params[1];
		BoxedStatements premise3 = (BoxedStatements) params[2];

		ParseTree p1LeftSubtree = premise1.getFormula().getParseTree()
				.getLeftSubtree();
		ProofStatement p2FirstStat = premise2.getFirst();
		if (p2FirstStat.getRule().type != Rule.ASSUMPTION) {
			throw new RuleException("The first formula " + p2FirstStat + " "
					+ "in box " + premise2.getRange() + " must be an "
					+ "assumption");
		}

		if (!p1LeftSubtree.equals(p2FirstStat.getFormula().getParseTree())) {
			throw new RuleException("The first formula " + p2FirstStat + " in "
					+ "the box: " + premise2.getRange() + " must match the "
					+ "formula on the left side of the main connective 'v' "
					+ "in " + premise1);
		}

		ParseTree p1RightSubtree = premise1.getFormula().getParseTree()
				.getRightSubtree();
		ProofStatement p3FirstStat = premise3.getFirst();
		if (p3FirstStat.getRule().type != Rule.ASSUMPTION) {
			throw new RuleException("The first formula " + p3FirstStat + " "
					+ "in box " + premise3.getRange() + " must be an "
					+ "assumption");
		}
		if (!p1RightSubtree.equals(p3FirstStat.getFormula().getParseTree())) {
			throw new RuleException("The first formula " + p3FirstStat + " in "
					+ "the box: " + premise3.getRange() + " must match the "
					+ "formula on the right side of the main connective 'v' "
					+ "in " + premise1);
		}

		ProofStatement conclusion = (ProofStatement) params[3];
		ProofStatement p2LastStat = premise2.getLast();
		ProofStatement p3LastStat = premise3.getLast();

		if (!(p2LastStat.getFormula().getParseTree().equals(
				p3LastStat.getFormula().getParseTree()) && p3LastStat
				.getFormula().getParseTree().equals(
						conclusion.getFormula().getParseTree()))) {

			throw new RuleException("The last formula " + p2LastStat + " in "
					+ "box: " + premise2.getRange() + " and the last "
					+ "formula " + p3LastStat + " in the box: "
					+ premise3.getRange() + " and the formula " + conclusion
					+ " outside the box must all be the same");
		}
	}
}