package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleSYM extends Rule {
	private static String name = getDescription(Rule.SYM);

	public RuleSYM() {
		super();
	}

	public int getType() {
		return Rule.SYM;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 2 || params[0] == null
				|| params[1] == null) {
			throw new RuleException("Rule requires a formula");
		} else if (!(params[0] instanceof ProofStatement)) {
			throw new RuleException("Rule requires a formula");
		}

		ProofStatement premise = (ProofStatement) params[0];
		ProofStatement conclusion = (ProofStatement) params[1];

		Symbol root = null;
		root = premise.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_EQUAL) {
			throw new RuleException("Formula " + premise
					+ " must have '=' as its main connective");
		}
		root = conclusion.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_EQUAL) {
			throw new RuleException("Formula " + conclusion
					+ " must have '=' as its main connective");
		}

		ParseTree pLSubtree = premise.getFormula().getParseTree()
				.getLeftSubtree();
		ParseTree pRSubtree = premise.getFormula().getParseTree()
				.getRightSubtree();

		ParseTree cLSubtree = conclusion.getFormula().getParseTree()
				.getLeftSubtree();
		ParseTree cRSubtree = conclusion.getFormula().getParseTree()
				.getRightSubtree();

		if (!pRSubtree.equals(cLSubtree)) {
			throw new RuleException(
					"The left side of the main connective '=' in " + "formula "
							+ conclusion + " must match the right side "
							+ "of the main connective '=' in formula "
							+ premise);
		} else if (!pLSubtree.equals(cRSubtree)) {
			throw new RuleException(
					"The right side of the main connective '=' in "
							+ "formula " + conclusion
							+ " must match the right side "
							+ "of the main connective '=' in formula "
							+ premise);
		}
	}
}